﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text.RegularExpressions;
using System.Text;

namespace DmisLib.Calc
{
    /// <summary>
    /// Class separates the standard cell address 'A1' to Col and Row number.
    /// Not needed after conversion from OpenOffice to Excel.
    /// Works for multiple letter columns.
    /// </summary>
    public class CellMap
    {
        public CellMap(String position)
        {
            parsePosition(position);
        }

        public CellMap(int column, int row)
        {
            ColNumber = column;
            RowNumber = row;
        }

        public int RowNumber
        { set; get; }

        public int ColNumber
        { set; get; }

        private void parsePosition(String position)
        {
            String pattern = @"([a-zA-Z]+)|([0-9]+)";

            MatchCollection Matches = Regex.Matches(position.ToUpper(),
                                                    pattern,
                                                    RegexOptions.IgnoreCase |
                                                    RegexOptions.IgnorePatternWhitespace);

            String[] labelParts = new String[Matches.Count];

            int i = 0;
            foreach (Match NextMatch in Matches)
            {
                labelParts[i] = NextMatch.Value;
                i++;
            }

            ColNumber = ConvertAlphaToNum(labelParts[0]);
            RowNumber = Convert.ToInt32(labelParts[1]) - 1;

        }

        private static int ConvertAlphaToNum(String col)
        {
            // Convert a character column label to a numerical value.
            Double colNum = 0.0;
            Double baseVal = 26.0;
            Int32 ascOffset = 65;
            Double multiplier = col.Length - 1;

            foreach (Char c in col)
            {
                colNum = colNum + (Math.Pow(baseVal, multiplier) * (Convert.ToInt32(c) - ascOffset)) + (baseVal * multiplier);
                multiplier--;
            }

            return (Int32)colNum;
        }
    }
}
