﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using DmisLib.Utilities;

namespace DmisLib.Dmis
{
    public abstract class StandardFolder
    {
        DirectoryInfo FolderInfo;
        private String FPath;

        public StandardFolder(String path)
        {
            FolderError = "";
            FolderIsValid = ValidateFolder(path);
        }

        public string FolderError { get; set; }
        public bool FolderIsValid { get; set; }
        public String FolderName { get; set; }
        public String FolderPath
        {
            get { return FPath; }
            set
            {
                if (!Directory.Exists(value)) ValidateFolder(value);
            }
        }


        /// <summary>
        /// Return a list of file names from the work folder that match the RegExp.
        /// </summary>
        /// <param name="filter">RegExp used to select files from work folder.</param>
        /// <returns></returns>
        public String[] getFilesByRegExp(String filter)
        {
            if (!FolderIsValid) return null;
            return Directory.GetFiles(FolderPath, filter);
        }

        public String[] getFileNamesByRegExp(String filter)
        {
            if (!FolderIsValid) return null;

            String[] p1List = getFilesByRegExp(filter);
            String[] fileNames = new String[p1List.Count()];

            int i = 0;
            foreach (String pname in p1List)
            {
                FileInfo info = new FileInfo(pname);
                String name = info.Name;
                fileNames[i] = StringTransforms.stripExtension(name);
                i++;
            }

            return fileNames;
        }

        public String[] getSubFolders()
        {
            if (!FolderIsValid) return null;
            DirectoryInfo dirInfo = new DirectoryInfo(FolderPath);
            DirectoryInfo[] dirList = dirInfo.GetDirectories();
            String[] FolderNames = new String[dirList.Count()];

            int i = 0;
            foreach (DirectoryInfo info in dirList)
            {
                FolderNames[i] = info.Name;
                i++;
            }

            return FolderNames;
        }

        protected Boolean ValidateFolder(String path)
        {
            if (path == "" || path == null)
            {
                FPath = "";
                FolderName = "";
                return false;
            }

            // Check to see if folder already exist
            FolderInfo = new DirectoryInfo(path);
            if (FolderInfo.Exists)
            {
                FPath = path;
                FolderName = FolderInfo.Name;
                return true;
            }

            // try to create the folder
            try
            {
                Directory.CreateDirectory(path);
            }
            catch (Exception ex)
            {
                FolderError = ex.Message;
                FPath = "";
                FolderName = "";
                FolderInfo = null;
                return false;
            }

            FPath = path;
            FolderName = FolderInfo.Name;
            return true;
        }

        /// <summary>
        /// Copy from another folder a list of files by regExpression
        /// </summary>
        /// <param name="remoteFolder"></param>
        /// <param name="wildCard"></param>
        /// <returns></returns>
        public int importFiles(String remoteFolder, String wildCard)
        {
            String[] fileList = Directory.GetFiles(remoteFolder, wildCard);
            if (fileList.Length > 0)
            {
                foreach (String remoteFile in fileList)
                {
                    FileInfo Vaxfile = new FileInfo(remoteFile);
                    String LocalFile = FolderPath + @"\" + Vaxfile.Name;
                    File.Delete(LocalFile);
                    File.Copy(remoteFile, LocalFile, true);
                }
            }

            return fileList.Length;
        }

        /// <summary>
        /// Delete files from folder by regexpression
        /// </summary>
        /// <param name="wildCard"></param>
        public void deletefiles(String wildCard)
        {
            // Delete previous files if any exist

            String[] oldFiles = Directory.GetFiles(FolderPath, wildCard);
            foreach (string oldfile in oldFiles)
            {
                File.Delete(oldfile);
            }
        }

    }
}
