﻿using System;
using System.Linq;
using DmisLib.Utilities;

namespace DmisLib.Session
{
    class DimensionFactory
    {
        DimensionData dimData;
        private static Int32 passNumber;
        private static string dimID;

        // Section DIMENSION: Methods which interact with the Dimension data table.
        public Boolean processDimension(PCDLRN.Command cmd, out DimensionData data)
        {
            PCDLRN.DimensionCmd DimCmd = cmd.DimensionCommand;
            string displayName = cmd.DisplayID;
            string featureName = DimCmd.Feat1;
            string dimType = cmd.Type.ToString();
            passNumber = 1;
            switch (cmd.Type)
            {
                case PCDLRN.OBTYPE.DIMENSION_2D_ANGLE:
                    startNewDimension(displayName, featureName);
                    break;
                case PCDLRN.OBTYPE.DIMENSION_2D_DISTANCE:
                    startNewDimension(displayName, featureName);
                    break;
                case PCDLRN.OBTYPE.DIMENSION_3D_ANGLE:
                    startNewDimension(displayName, featureName);
                    break;
                case PCDLRN.OBTYPE.DIMENSION_3D_DISTANCE:
                    startNewDimension(displayName, featureName);
                    break;
                case PCDLRN.OBTYPE.DIMENSION_ANGULARITY:
                    startNewDimension(displayName, featureName);
                    break;
                case PCDLRN.OBTYPE.DIMENSION_COAXIALITY:
                    startNewDimension(displayName, featureName);
                    break;
                case PCDLRN.OBTYPE.DIMENSION_CONCENTRICITY:
                    startNewDimension(displayName, featureName);
                    break;
                case PCDLRN.OBTYPE.DIMENSION_PARALLELISM:
                    startNewDimension(displayName, featureName);
                    break;
                case PCDLRN.OBTYPE.DIMENSION_FLATNESS:
                    startNewDimension(displayName, featureName);
                    break;
                case PCDLRN.OBTYPE.DIMENSION_PERPENDICULARITY:
                    startNewDimension(displayName, featureName);
                    break;
                case PCDLRN.OBTYPE.DIMENSION_PROFILE:
                    startNewDimension(displayName, featureName);
                    break;
                case PCDLRN.OBTYPE.DIMENSION_ROUNDNESS:
                    startNewDimension(displayName, featureName);
                    break;
                case PCDLRN.OBTYPE.DIMENSION_ROUNDNESS_LOCATION:
                    startNewDimension(displayName, featureName);
                    break;
                case PCDLRN.OBTYPE.DIMENSION_START_LOCATION:
                    startNewDimension(displayName, featureName);
                    passNumber = 0;
                    break;
                case PCDLRN.OBTYPE.DIMENSION_STRAIGHTNESS:
                    startNewDimension(displayName, featureName);
                    break;
                case PCDLRN.OBTYPE.DIMENSION_SYMMETRY:
                    startNewDimension(displayName, featureName);
                    break;
                case PCDLRN.OBTYPE.DATDEF_COMMAND:
                    startNewDimension(displayName, featureName);
                    break;
                default:
                    break;
            }

            if (passNumber == 0)
            {
                data = null;
                return false;
            }

            extractDimensionData(DimCmd, dimType);
            data = dimData;

            if (dimData != null)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        private void startNewDimension(string cmdID, string cmdFeat)
        {
            // If either the command name or feature reference is part of an array this method separates
            // the array index value.
            Int32 cmdIndex;
            StripIndex(cmdID, out dimID, out cmdIndex);

            string featName;
            Int32 featIndex;
            StripIndex(cmdFeat, out featName, out featIndex);

            // create the new non-Pcdmis Dimension object to hold the Pcdmis Dimension data
            dimData = new DimensionData(dimID, cmdIndex, featName, featIndex);
        }

        private Boolean StripIndex(string name, out string shortName, out Int32 index)
        {
            // Pcdmis program is responsible for the naming of the point and dimension
            // This routine strips the index from the Pcdmis array name and returns the name and the
            // index number as two separate entities.
            shortName = name;
            index = -1;
            if (!name.Contains('[')) return false;

            Int32 pos1 = name.IndexOf('[');
            Int32 pos2 = name.IndexOf(']');
            index = Convert.ToInt32(name.Substring(pos1 + 1, pos2 - pos1 - 1));
            shortName = name.Substring(0, pos1);
            return true;
        }

        private void extractDimensionData(PCDLRN.DimensionCmd DimCmd, string DimType)
        {
            if (dimData != null)
            {
                dimData.DimensionType = DimType;
                dimData.Id = dimID + "_" + DimCmd.AxisLetter;
                dimData.Axis = DimCmd.AxisLetter;
                dimData.Nominal = DimCmd.NOMINAL;
                dimData.Measured = DimCmd.Measured;
                dimData.PlusTol = DimCmd.Plus;
                dimData.MinusTol = DimCmd.Minus;
                dimData.Deviation = DimCmd.Deviation;
                dimData.OutTol = DimCmd.OutTol;
                dimData.TimeStamp = DmisDateTime.TimeStamp(DateTime.Now);
            }
        }
    }
}
